#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <ctype.h>
#include <assert.h>


typedef struct {
    char *nom;
    uint32_t taille;
    uint32_t offset;
} infofic;



void Write32BE(void *p, uint32_t v)
{
    uint8_t *q = (uint8_t *)(p) + 3;

    *q-- = v;
    v >>= 8;
    *q-- = v;
    v >>= 8;
    *q-- = v;
    v >>= 8;
    *q = v;
}


void EcritBufFic(FILE *f, void *buf, size_t taille)
{
    assert(fwrite(buf, 1, taille, f) == taille);
}


void EcritInt32Fic(FILE *f, uint32_t i)
{
    uint8_t buf[4];
    
    Write32BE(buf, i);
    EcritBufFic(f, buf, sizeof(buf));
}


void EcritZeroFic(FILE *f, uint32_t nb)
{
    void *buf;
    
    assert((buf = malloc(nb)) != NULL);
    memset(buf, 0, nb);
    EcritBufFic(f, buf, nb);
    free(buf);
}


uint32_t TailleFic(char *nomfic)
{
    FILE *f;
    uint32_t taille;
    
    assert((f = fopen(nomfic, "rb")) != NULL);
    assert(fseek(f, 0, SEEK_END) == 0);
    assert((taille = ftell(f)) != (unsigned)-1);
    assert(fclose(f) == 0);

    return taille;
}


int main(int argc, char *argv[])
{
    uint32_t nbfics;
    infofic *tabfics;
    FILE *fimage;

    if (argc < 2)
    {
        printf("Utilisation : ImageSFV [fichier] [fichier] ... [fichier] <fichier image>\n");
        exit(0);
    }

    nbfics = argc - 2;
    assert((tabfics = calloc(nbfics, sizeof(infofic))) != NULL);
    {
        uint32_t i = 1;
        uint32_t n = nbfics;
        infofic *p = tabfics;
        uint32_t offset = (nbfics + 1) * 4;
        
        while (n--)
        {
            p->nom = argv[i];
            p->taille = TailleFic(p->nom);
            p->offset = offset;
            offset += p->taille;

            i++;
            p++;
        }
    }
    
    assert((fimage = fopen(argv[argc - 1], "wb")) != NULL);

    {
        uint32_t n = nbfics;
        infofic *p = tabfics;

        while (n--)
        {
            EcritInt32Fic(fimage, p->offset);
            p++;
        }
        
        EcritInt32Fic(fimage, (uint32_t)-1);
    }
    
    {
        uint32_t n = nbfics;
        infofic *p = tabfics;

        while (n--)
        {
            FILE *f;
            void *buf;

            assert((buf = malloc(p->taille)) != NULL);
            assert((f = fopen(p->nom, "rb")) != NULL);
            assert(fread(buf, 1, p->taille, f) == p->taille);
            EcritBufFic(fimage, buf, p->taille);
            assert(fclose(f) == 0);
            free(buf);
            
            p++;
        }
    }

    assert(fclose(fimage) == 0);
    
    free(tabfics);

    return 0;
}
